/*

Lostpod Widget v1.0

Copyright (C) 2006, Andrea Carolfi - All rights reserved.
Code: Andrea Carolfi - http://www.hurricane-studios.com

1.0 - 25/09/06 - First release

*/

// Global variables for the info and done buttons; keeping these around lets us modify them later
var glassDoneButton;
var blackInfoButton;

var feed = {title:"Lostpod Widget", top:"Lostpod Widget", bottom:"IT", url:"http://www.lostpod.it/lostpod.xml"};			
var website = {title:"Lostpod", top:"Lostpod", bottom:"IT", url:"http://www.lostpod.it"};			
	
var scrollArea, scrollbar;

//var selURL;

var last_updated = 0;
var xml_request = null;

//---------------------------------------------------------------------------------------------
//
// setup() - is the onload handler for the widget. It calls constructors for the AppleInfoButton,
//           AppleGlassButton, AppleVerticalScrollbar and AppleScrollArea classes that populate
//           existing divs with their respective controls.
//
//----------------------------------------------------------------------------------------------
function setup ()
{
	glassDoneButton = new AppleGlassButton(document.getElementById("doneButton"),
		"Ok", hidePrefs);
	blackInfoButton = new AppleInfoButton(document.getElementById("infoButton"),
		document.getElementById("front"), "black", "black", showPrefs);

	scrollbar = new AppleVerticalScrollbar(document.getElementById("myScrollBar"));
	scrollArea = new AppleScrollArea(document.getElementById("contents"), scrollbar);
	scrollArea.scrollsHorizontally = false;
	scrollArea.singlepressScrollPixels = 15;
	
	window.onfocus = function () { scrollArea.focus(); }
	window.onblur = function () { scrollArea.blur(); }
	
	if (!window.widget)
	{
		show ();
	}
}

//---------------------------------------------------------------------------------------------
//
// show - post a request to get Lostpod RSS podcasts when showing the widget. Space requests
//		  by at least 15 minutes to avoid hitting the server too often. Also, note that you
//		  should cancel any outstanding request before posting the new one. 
//
//----------------------------------------------------------------------------------------------
function show ()
{
	var now = (new Date).getTime();
	
	// only check if 15 minutes have passed
	if ((now - last_updated) > 900000)
	{
		if (xml_request != null)
		{
			xml_request.abort();
			xml_request = null;
		}
		xml_request = new XMLHttpRequest();

		xml_request.onload = function(e) {xml_loaded(e, xml_request);}
		xml_request.overrideMimeType("text/xml");
		xml_request.open("GET", feed.url);
		xml_request.setRequestHeader("Cache-Control", "no-cache");
		xml_request.send(null);
    }
}

if (window.widget)
{
	widget.onshow = show;
}

//---------------------------------------------------------------------------------------------
//
// findChild - scan the children of a given DOM element for a node matching nodeName; much more
//				efficient than the standard DOM methods (getElementsByTagName, etc) if you know
//				what you're looking for
//
//----------------------------------------------------------------------------------------------
function findChild (element, nodeName)
{
	var child;
	
	for (child = element.firstChild; child != null; child = child.nextSibling)
	{
		if (child.nodeName == nodeName)
			return child;
	}
	
	return null;
}

//---------------------------------------------------------------------------------------------
//
// xml_loaded - extract the content of Lostpod RSS feed and place the items data into a
//			    results array: extract the title, enclosure tag and publication date for each item. 
//
//----------------------------------------------------------------------------------------------
function xml_loaded (e, request) 
{
	xml_request = null;
	if (request.responseXML)
	{		
		var contents = document.getElementById('contents');
		while (contents.hasChildNodes())
		{
			contents.removeChild(contents.firstChild);
		}
		
		// Get the top level <rss> element 
		var rss = findChild(request.responseXML, 'rss');
		if (!rss) {
			alert("no <rss> element"); 
			document.getElementById("ohnoes").style.display = "block";
			return;
		}
		
		// Get single subordinate channel element
		var channel = findChild( rss, 'channel');
		if (!channel) {
			alert("no <channel> element");
			document.getElementById("ohnoes").style.display = "block";
			return;
		}
		
		
		document.getElementById("ohnoes").style.display = "none";

		var results = new Array;
		
		// Get all item elements subordinate to the channel element
		// For each element, get title, link and publication date. 
		// Note that all elements of an item are optional. 
		for( var item = channel.firstChild; item != null; item = item.nextSibling)
		{
			if( item.nodeName == 'item' ) 
			{
				var title = findChild (item, 'title');
			
				// we have to have the title to include the item in the list 
				if( title != null ) 
				{
					var link = findChild (item, 'enclosure').url;
					var pubDate = findChild (item, 'pubDate');
					results[results.length] = {title:title.firstChild.data, 
						link:link, 
						date:new Date(Date.parse(pubDate.firstChild.data))
				   };
				}
			}
		}
		
		// sort by date
		results.sort (compFunc);
		
		// copy title and date into rows for display. Store link so it can be used when user
		// clicks on title
		nItems = results.length;
		var even = true;		
				
		for (var i = 0; i < nItems; ++i)
		{
			var item = results[i];
			var row = createRow (item.title, item.link, item.date, even);
			even = !even;
			
			contents.appendChild (row);
		}

		// update the scrollbar so scrollbar matches new data
		scrollArea.refresh();
		
		// set last_updated to the current time to keep track of the last time a request was posted
		last_updated = (new Date).getTime();
	}
}

//---------------------------------------------------------------------------------------------
//
// sortFunc - compare function used for sorting dates
//
//----------------------------------------------------------------------------------------------
function compFunc (a, b)
{
	if (a.date < b.date)
		return 1;
	else if (a.date > b.date)
		return -1;
	else
		return 0;
}

//---------------------------------------------------------------------------------------------
//
// createRow - add data to the next row in the widget body. Rows have alternating (light and 
//			   dark backgound). The title and date as displayed for each item. The link is used
//			   when the user clicks on a RSS title. 
//
//----------------------------------------------------------------------------------------------
function createRow (title, link, date, even)
{
	var row = document.createElement ('div');
	row.setAttribute ('class', 'row ' + (even ? 'light' : 'dark'));
	
	var title_div = document.createElement ('div');
	title_div.innerText = title;
	title_div.setAttribute ('class', 'title');
	if (link != null)
	{
		title_div.setAttribute ('the_link', link);
		title_div.setAttribute ('onclick', 'clickOnTitle (event, this);');
	}
	row.appendChild (title_div);
	
	if (date != null)
	{
		var date_div = document.createElement ('div');
		date_div.setAttribute ('class', 'date');
		date_div.innerText = createDateStr (date);
		
		row.appendChild (date_div);
	}
	
	return row;	
}

//---------------------------------------------------------------------------------------------
//
// createDateStr - create a string version of a given date. Actually is not localized.
//
//----------------------------------------------------------------------------------------------
function createDateStr (date)
{
	var month;
	switch (date.getMonth())
	{
		case 0: month = 'Jan'; break;
		case 1: month = 'Feb'; break;
		case 2: month = 'Mar'; break;
		case 3: month = 'Apr'; break;
		case 4: month = 'May'; break;
		case 5: month = 'Jun'; break;
		case 6: month = 'Jul'; break;
		case 7: month = 'Aug'; break;
		case 8: month = 'Sep'; break;
		case 9: month = 'Oct'; break;
		case 10: month = 'Nov'; break;
		case 11: month = 'Dec'; break;
	}	
	return month + ' ' + date.getDate();
}

//---------------------------------------------------------------------------------------------
//
// clickOnTitle - load podcast selected by the user when he clicks on an article's title.  
//
//----------------------------------------------------------------------------------------------
function clickOnTitle (event, div)
{
	if (window.widget)
	{
		pl = document.getElementById("player");
		pl.SetURL(div.the_link);
		pl.Play();
	}
	else
	{
		pl = document.getElementById("player");
		pl.SetURL(div.the_link);
		pl.Play();
	}
}

//---------------------------------------------------------------------------------------------
//
// clickOnWidgetTitle - take user to the main web page when he clicks on the widget title. 
//
//----------------------------------------------------------------------------------------------
function clickOnWidgetTitle (event)
{
	if (window.widget)
	{
			widget.openURL (website.url);
	} else document.location = website.url;
}

//function selected()
//{
//	return selURL;
//}

//---------------------------------------------------------------------------------------------
//
// openUrl - open Safari and go to the url given
//
//----------------------------------------------------------------------------------------------
function openUrl(url)
{
	if (window.widget)
	{
		widget.openURL(url);
	}
}

/*********************************/
// HIDING AND SHOWING PREFERENCES
/*********************************/

//---------------------------------------------------------------------------------------------
//
// showPrefs() - is called when the preferences flipper is clicked upon.  It freezes the front
//               of the widget, hides the front div, unhides the back div, and then flips the
//               widget over.
//
//----------------------------------------------------------------------------------------------
function showPrefs ()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	var player = document.getElementById("playercontainer");
	
	if (window.widget)
		widget.prepareForTransition("ToBack");		// freezes the widget so that you can change it without the user noticing
	
	front.style.display="none";		// hide the front
	back.style.display="block";		// show the back
	player.style.visibility="hidden";// hide the player
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget over	

}

//---------------------------------------------------------------------------------------------
//
// hidePrefs() - is called by the done button on the back side of the widget.  It performs the
//               opposite transition as showPrefs() does.
//
//----------------------------------------------------------------------------------------------
function hidePrefs ()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	var player = document.getElementById("playercontainer");
	
	if (window.widget)
		widget.prepareForTransition("ToFront");		// freezes the widget and prepares it for the flip back to the front
	
	back.style.display="none";			// hide the back
	front.style.display="block";		// show the front
	player.style.visibility="show";		// show the player
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget back to the front
}
